import threading
from collections import deque

class await_fifo:
	# this class has purpose handling a streaming like response
	# as single response object not as multiple bytes stream
	# it is questionable about efficience 
	# but it makes possible optimization response contents - compressing, http2 framing etc - delivery but just bytes relaying
		
	def __init__ (self):
		self.l = deque ()
		self.has_None = False
	
	def working (self):
		return (self.has_None or self.l) and 1 or 0
	
	def __len__ (self):
		for i in range (len (self.l)):
			try:
				readyfunc = getattr (self.l [0], 'ready')
			except AttributeError:
				return 1
				
			if readyfunc ():
				return 1
				
			self.l.rotate (-1)
		
		if self.has_None and not self.l:
			# for return None
			self.l.append (None)
			self.has_None = False
			return 1
			
		return 0
			
	def __getitem__(self, index):
		return self.l [index]
				
	def __setitem__(self, index, item):
		self.l [index] = item
		
	def __delitem__ (self, index):
		del self.l [index]
	
	def append (self, item):
		self.insert (-1, item)
	
	def appendleft (self, item):
		self.insert (0, item)
		
	def insert (self, index, item):
		if item is None:
			self.has_None = True
			return
			
		if self.has_None and index != 0:
			return # deny adding
		
		if not self.l:
			return self.l.append (item)
				
		if index == -1:	
			return self.l.append (item)
		
		try:
			readyfunc = getattr (item, 'ready')
		except AttributeError:
			pass
		else:	
			if not readyfunc ():
				return self.l.append (item)
				
		return self.l.appendleft (item)
		
	def clear (self):
		self.l.clear ()
		self.has_None = False
	
	
class await_ts_fifo (await_fifo):
	# HTTP/1.x needn't this class, because one channel handles only one request
	# this will be used for handling multiple requests like HTTP/2
	
	def __init__ (self):
		await_fifo.__init__ (self)
		self._lock = threading.Lock ()
	
	def working (self):
		with self._lock:
			return await_fifo.working (self)
		
	def __len__ (self):
		with self._lock:
			return await_fifo.__len__ (self)
			
	def __getitem__(self, index):
		with self._lock:
			return await_fifo.__getitem__ (self, index)
		
	def __setitem__(self, index, item):
		with self._lock:
			await_fifo.__setitem__ (self, index, item)
		
	def __delitem__ (self, index):
		with self._lock:
			await_fifo.__delitem__ (self, index)
	
	def clear (self):
		with self._lock:
			await_fifo.clear (self)
			
	def insert (self, index, item):
		with self._lock:
			await_fifo.insert (self, index, item)



import boto.ec2

def get_instances_by_auto_scaling_group (name, status = None):
	return get_instances_by_tag ("aws:autoscaling:groupName", name, status)	

def get_instances_by_tag (name, key = None, status = None, region = 'us-east-1'):
	fl = {'tag:' + name: key}
	conn = boto.ec2.connect_to_region (region)
	reservations = conn.get_all_instances (filters = fl)
	if not reservations:
		return []
	instances = [r.instances [0] for r in reservations]
	if status:
		return [i for i in instances if i.update () == status]
	return instances

def get_instance_by_id (instance_id, region = 'us-east-1'):
	conn = boto.ec2.connect_to_region (region)
	reservations = conn.get_all_instances ([instance_id])
	if not reservations:
		return []
	instances = reservations [0].instances
	return instances [0]



import threading
import asyncore

class thread_safe_socket_map (dict):
	def __init__ (self):
		self.lock = threading.RLock ()
	
	def __setitem__ (self, k, v):	
		with self.lock:
			dict.__setitem__ (self, k, v)		
	
	def __contains__ (self, k):	
		with self.lock:
			v = dict.__contains__ (self, k)
		return v
		
	def __len__ (self):	
		with self.lock:
			v = dict.__len__ (self)
		return v
	
	def __nonzero__(self):
		with self.lock:
			v = dict.__len__ (self)
		return v
			
	def __getitem__ (self, k):	
		with self.lock:
			v = dict.__getitem__ (self, k)
		return v
		
	def __delitem__ (self, k):
		with self.lock:
			dict.__delitem__ (self, k)	
	
	def has_key (self, k):
		with self.lock:
			v = dict.has_key (self, k)			
		return v
			
	def get (self, k, d = None):
		with self.lock:
			v = dict.get (self, k, d)
		return v
	
	def popitem (self, k):
		with self.lock:
			v = dict.popitem (self, k)
		return v
		
	def items (self):
		with self.lock:
			v = dict.items (self)		
		return v
			
	def keys (self):
		with self.lock:
			v = dict.keys (self)
		return v
	
	def values (self):
		with self.lock:
			v = dict.values (self)		
		return v


if not hasattr (asyncore, "_socket_map"):
	asyncore._socket_map = asyncore.socket_map
	del asyncore.socket_map
	asyncore.socket_map = thread_safe_socket_map ()
	


import os
import sys
import time
import signal
import getopt
from . import killtree, processutil
from .. import pathtool

class Daemonizer:
	def __init__(self, chdir="/", procname = None, umask=0o22):
		self.chdir = chdir
		self.procname = procname
		self.umask = umask
		self.pidfile = os.path.join (chdir, '.pid')
	
	def runAsDaemon(self):
		if status (self.chdir, self.procname):
			return 0
						
		self.fork_and_die()
		self.dettach_env ()		
		self.fork_and_die()	
		
		sys.stdout.flush()
		sys.stderr.flush()
		self.attach_stream('stdin', 'r')
		self.attach_stream('stdout', 'a+')
		self.attach_stream('stderr', 'a+')
		return 1
		
	def dettach_env (self):
		os.setsid()
		os.umask(self.umask)
		os.chdir(self.chdir)
	
	def attach_stream (self, name, mode, fd = '/dev/null'):
		stream = open(fd, mode)
		os.dup2(stream.fileno(), getattr(sys, name).fileno())
	
	def fork_and_die(self):
		r = os.fork()
		if r == -1:
			raise OSError("Couldn't fork().")
		elif r > 0:  # I'm the parent
			if self.pidfile: 
				open (self.pidfile, 'w').write (str(r))
			sys.exit(0)
		elif r < 0:
			raise OSError("Something bizarre happened while trying to fork().")
			# now only r = 0 (the child) survives.
		return r

def status (chdir, procname = None):
	pidfile =  os.path.join (chdir, '.pid')
	if not os.path.isfile (pidfile):		
		return 0
	with open (pidfile) as f:
		pid = int (f.read ())
	return processutil.is_running (pid, procname) and pid or 0

def kill (chdir, procname = None, include_children = True, signaling = True):
	import psutil
	
	for i in range (2):
		pid = status (chdir, procname)		
		if not pid:	
			break
		
		if signaling:	
			os.kill (pid, signal.SIGTERM)
			time.sleep (2)
			
		if include_children:
			try:
				killtree.kill (pid, True)
			except psutil.NoSuchProcess:
				pass	
		
		while processutil.is_running (pid, procname):
			time.sleep (1)
			
	try:
		os.remove (os.path.join (chdir, ".pid"))
	except FileNotFoundError:	
		pass

def handle_commandline (sopt, lopt, working_dir, procname):
	def _start (working_dir, procname):
		if not Daemonizer (working_dir, procname).runAsDaemon ():
			print ("already running")
			sys.exit ()

	def _stop (working_dir, procname):
		kill (working_dir, procname, True)
	
	def _status (working_dir, procname):
		pid = status (working_dir, procname)
		if pid:
			print ("running [%d]" % pid)
		else:
			print ("stopped")
	
	def _restart (working_dir, procname):
		_stop (working_dir, procname)
		time.sleep (2)
		_start (working_dir, procname)
	
	if "d" in sopt:
		raise SystemError ("Short option -d is reserved")	
	sopt += "d"
	
	pathtool.mkdir (working_dir)
	argopt = getopt.getopt(sys.argv[1:], sopt, lopt)		
	daemonics = [] 
	arglist = []
	for arg in argopt [1]:
	 	if arg in ("start", "restart", "stop", "status"):
	 		daemonics.append (arg)
	 	else:
	 		arglist.append (arg)
	argopt = (argopt [0], arglist)
	
	for k, v in argopt [0]:
		if k == "-d" or "start" in daemonics: 
			_start (working_dir, procname)			
		elif "restart" in daemonics: 
			_restart (working_dir, procname)			
		elif "stop" in daemonics:
			_stop (working_dir, procname)
			sys.exit ()			
		elif "status" in daemonics:		
			_status (working_dir, procname)
			sys.exit ()
		
	return argopt

if __name__ == "__main__"	:
	import time
	
	Daemonizer ().runAsDaemon ()	
	f = open ('/home/ubuntu/out', 'w')
	while 1:
		time.sleep (1)
		f.write ('asdkljaldjalkdjalkdsa\n')
		f.flush()
	f.close ()

		
